import time
import logging

from srsgui import Task
from srsgui import BoolInput, IntegerListInput, FloatListInput, InstrumentInput, CommandInput
from srsinst.sr860 import SR860, get_sr860

from srsinst.sr860.instruments.components import Aux
from srsinst.sr860.instruments.keys import Keys


class AuxOutputTask(Task):
    """
    When the task is selected, it reads the current auxiliary output values from the unit. \
Adjust the output values from the input panel, and press the adjust button. \
It will change the aux output values, even without running the task.
    """

    InstName = 'inst to change'
    Ch1 = 'channel 1 (V)'
    Ch2 = 'channel 2 (V)'
    Ch3 = 'channel 3 (V)'
    Ch4 = 'channel 4 (V)'

    input_parameters = {
        InstName: InstrumentInput(),
        Ch1:      CommandInput(f'aux.output[0]', Aux.output),
        Ch2:      CommandInput(f'aux.output[1]', Aux.output),
        Ch3:      CommandInput(f'aux.output[2]', Aux.output),
        Ch4:      CommandInput(f'aux.output[3]', Aux.output),
    }

    def setup(self):
        self.logger = logging.getLogger(__file__)
        self.lockin = get_sr860(self, self.get_input_parameter(self.InstName))

    def test(self):

        self.params = self.get_all_input_parameters()
        self.logger.info(self.params)
        self.logger.info('Set aux outputs')

    def cleanup(self):
        pass



import time
import logging

from srsgui import Task
from srsgui import BoolInput, IntegerListInput, FloatListInput, InstrumentInput, CommandInput
from srsinst.sr860 import SR860, get_sr860

from srsinst.sr860.instruments.components import Signal, Reference
from srsinst.sr860.instruments.keys import Keys


class ConfigureReferenceTask(Task):
    """
When this task runs, it sets the commands in Reference component, \
reads the relevant parameters from the unit, and updates the input panel display. \
To change parameter values, press the Apply button.

You change parameters to the values you want, and press the Apply button. \
After it updates the parameters in the unit, and finishes the task.
    """
    InstName = 'inst to change'
    TimebaseMode = 'timebase mode'
    TimebaseSource = 'timebase source'
    InternalFrequency = 'internal F (Hz)'
    Phase = 'phase (degree)'
    Harmonics = 'harmonics'
    HarmonicsDual = 'harmonics dual'
    BladeSlots = 'blade slots (/rev.)'
    BladePhase = 'blade phase (degree)'
    SineAmplitude = 'sine amplitude (V)'
    SineOffset = 'sine dc level (V)'
    SineDCMode = 'sine dc mode (V)'
    ReferenceSource = 'reference source'
    TriggerMode = 'trigger mode'
    TriggerInput = 'trigger input'

    input_parameters = {
        InstName:       InstrumentInput(),
        TimebaseMode:   CommandInput('ref.timebase_mode', Reference.timebase_mode),
        TimebaseSource: CommandInput('ref.timebase_source', Reference.timebase_source),
        InternalFrequency: CommandInput('ref.internal_frequency', Reference.internal_frequency),
        Phase: CommandInput('ref.phase', Reference.phase),
        Harmonics: CommandInput('ref.harmonics', Reference.harmonics),
        HarmonicsDual: CommandInput('ref.harmonics_dual', Reference.harmonics_dual),
        BladeSlots: CommandInput('ref.blade_slots', Reference.blade_slots),
        BladePhase: CommandInput('ref.blade_phase', Reference.blade_phase),
        SineAmplitude: CommandInput('ref.sine_out_amplitude', Reference.sine_out_amplitude),
        SineOffset: CommandInput('ref.sine_out_offset', Reference.sine_out_offset),
        SineDCMode: CommandInput('ref.sine_out_dc_mode', Reference.sine_out_dc_mode),
        ReferenceSource: CommandInput('ref.reference_source', Reference.reference_source),
        TriggerMode: CommandInput('ref.trigger_mode', Reference.trigger_mode),
        TriggerInput: CommandInput('ref.trigger_input', Reference.trigger_input),
    }

    def setup(self):
        self.logger = logging.getLogger(__file__)
        self.lockin = get_sr860(self, self.get_input_parameter(self.InstName))

    def test(self):
        self.logger.info('Current Reference parameters')
        self.logger.info(self.get_all_input_parameters())

    def cleanup(self):
        pass




import socket
from struct import unpack_from
import numpy as np

from srsgui.inst.component import Component
from srsgui.inst.commands import Command, GetCommand,\
                                 BoolCommand, BoolGetCommand,\
                                 IntCommand, IntGetCommand, IntSetCommand,\
                                 FloatCommand, FloatSetCommand, FloatGetCommand, \
                                 DictCommand, DictGetCommand

from srsgui.inst.indexcommands import IndexCommand, IndexGetCommand, \
                                      IntIndexCommand, IntIndexGetCommand, \
                                      BoolIndexCommand, BoolIndexGetCommand,\
                                      FloatIndexCommand, FloatIndexGetCommand, \
                                      DictIndexCommand
from .keys import Keys


class Reference(Component):
    TimebaseModeDict = {
        Keys.Auto: 0,
        Keys.Internal: 1
    }
    TimebaseSourceDict = {
        Keys.External: 0,
        Keys.Internal: 1
    }
    BladeSlotsDict = {
        6: 0,
        30: 1
    }
    SineOutDCModeDict = {
        Keys.Common: 0,
        Keys.Difference: 1
    }
    ReferenceSourceDict = {
        Keys.Internal: 0,
        Keys.External: 1,
        Keys.Dual: 2,
        Keys.Chop: 3
    }
    TriggerModeDict = {
        Keys.Sine: 0,
        Keys.PisitiveTTL: 1,
        Keys.NegativeTTL: 2
    }
    TriggerInputDict = {
        Keys.R50Ohms: 0,
        Keys.R1Meg: 1
    }
    timebase_mode = DictCommand('TBMODE', TimebaseModeDict)
    timebase_source = DictCommand('TBSTAT', TimebaseSourceDict)
    
    phase = FloatCommand('PHAS', unit='deg',min=-360000, max=360000, step=0.01,
                                 fmt='{:6e}', default_value=0.0)

    frequency = FloatCommand('FREQ', 'Hz', 0.001, 500000.0, 0.0001,)
    internal_frequency = FloatCommand('FREQINT','Hz', 0.001, 500000.0, 0.01)
    external_frequency = FloatGetCommand('FREQEXT')
    detection_frequency = FloatGetCommand('FREQDET')
    
    harmonics = IntCommand('HARM', '', 1, 99)
    harmonics_dual = IntCommand('HARMDUAL', '', 1, 99)
    
    blade_slots = DictCommand('BLADESLOTS', BladeSlotsDict)
    blade_phase = FloatCommand('BLADEPHASE')
    
    sine_out_amplitude = FloatCommand('SLVL')
    sine_out_offset = FloatCommand('SOFF')
    sine_out_dc_mode = DictCommand('REFM', SineOutDCModeDict)
    reference_source = DictCommand('RSRC', ReferenceSourceDict)
    
    trigger_mode = DictCommand('RTRG', TriggerModeDict)
    trigger_input = DictCommand('REFZ', TriggerInputDict)

    frequency_preset = FloatIndexCommand('PSTF', 3)
    sine_out_amplitude_preset = FloatIndexCommand('PSTA', 3)
    sine_out_offset_preset = FloatIndexCommand('PSTL', 3)

    exclude_capture = [harmonics, harmonics_dual]

    def auto_phase(self):
        self.comm.send('APHS')


class Signal(Component):
    InputModeDict = {
        Keys.Voltage: 0,
        Keys.Current: 1
    }
    VoltageInputModeDict = {
        Keys.AOnly: 0,
        Keys.AMinusB: 1
    }
    VoltageInputCouplingDict = {
        Keys.AC: 0,
        Keys.DC: 1
    }
    VoltageInputShieldDict = {
        Keys.Float:  0,
        Keys.Ground: 1
    }
    VoltageInputRangeDict = {
        1.0:  0,
        0.3:  1,
        0.1:  2,
        0.03: 3,
        0.01: 4
    }
    CurrentInputGainDict = {
        1e6: 0,
        1e8: 1
    }
    VoltageSensitivityDict = {1.0: 0}
    CurrentSensitivityDict = {1e-6: 0}
    index = 0
    for i in range(9):
        for j in [5.0, 2.0, 1.0]:
            index += 1
            VoltageSensitivityDict[float(f'{(j * 10 ** -(i + 1)):.1e}')] = index
            CurrentSensitivityDict[float(f'{(j * 10 ** -(i + 7)):.1e}')] = index

    TimeConstantDict = {}
    index = 0
    for i in range(11):
        for j in [1.0, 3.0]:
            TimeConstantDict[float(f'{(j * 10 ** (i - 6)):.1e}')] = index
            index += 1

    FilterSlopeDict = {
        6:  0,
        12: 1,
        18: 2,
        24: 3
    }
    OffOnDict = {'Off': 0,
                 'On':  1
    }

    input_mode = DictCommand('IVMD', InputModeDict)
    
    voltage_input_mode = DictCommand('ISRC', VoltageInputModeDict)
    voltage_input_coupling = DictCommand('ICPL', VoltageInputCouplingDict)

    voltage_input_shield = DictCommand('IGND', VoltageInputShieldDict)
    voltage_input_range = DictCommand('IRNG', VoltageInputRangeDict)
    voltage_sensitivity = DictCommand('SCAL', VoltageSensitivityDict)
    
    current_input_gain = DictCommand('ICUR', CurrentInputGainDict, unit='Ohm', fmt='{:0e}')
    current_sensitivity = DictCommand('SCAL', CurrentSensitivityDict, unit='A', fmt='{:0e}')
    time_constant = DictCommand('OFLT', TimeConstantDict, unit='s', fmt='{:0e}')

    strength_indicator = IntGetCommand('ILVL')
    filter_slope = DictCommand('OFSL', FilterSlopeDict, unit='dB/oct')
    sync_filter = DictCommand('SYNC', OffOnDict)
    advanced_filter = DictCommand('ADVFILT', OffOnDict)
    equivalent_noise_bandwidth = FloatCommand('ENBW')


class Output(Component):
    TypeDict = {
        Keys.XY: 0,
        Keys.RT: 1
    }
    IndexDict = {
        Keys.X: 0,
        Keys.Y: 1,
        Keys.R: 2
    }
    ChannelDict = {
        Keys.OutputCh1: 0,
        Keys.OutputCh2: 1
    }
    ExpandDict = {
        1:   0,
        10:  1,
        100: 2
    }

    type = DictIndexCommand('COUT', TypeDict, 1, 0, ChannelDict)
    expand = DictIndexCommand('CEXP', ExpandDict, 2, 0, IndexDict)
    offset = BoolIndexCommand('COFA', 2, 0, IndexDict)
    offset_percent = FloatIndexCommand('COFP', 2, 0, IndexDict)
    ratio_mode = BoolIndexCommand('CRAT', 2, 0, IndexDict)

    def auto_offset(self, index):
        self.comm.send('OAUT {}'.format(index))


class Aux(Component):
    ChannelDict = {
        Keys.Channel1: 0,
        Keys.Channel2: 1,
        Keys.Channel3: 2,
        Keys.Channel4: 3
    }

    input = FloatIndexGetCommand('OAUX', 3, 0, ChannelDict)
    output = FloatIndexCommand('AUXV', 3, 0, ChannelDict)


class Auto(Component):
    def set_phase(self):
        self.comm.send('APHS')

    def set_range(self):
        self.comm.send('ARNG')

    def set_scale(self):
        self.comm.send('ASCL')


class Display(Component):
    LayoutDict = {
        Keys.Trend:       0,
        Keys.History:     1,
        Keys.BarHistory: 2,
        Keys.FFT:         3,
        Keys.BarFFT:     4,
        Keys.Bar8:       5
    }
    ChannelDict = {
        Keys.Data1: 0,
        Keys.Data2: 1,
        Keys.Data3: 2,
        Keys.Data4: 3
    }
    ParameterDict = {
        Keys.X: 0,
        Keys.Y: 1,
        Keys.R: 2,
        Keys.Theta: 3,
        Keys.AuxInput1: 4,
        Keys.AuxInput2: 5,
        Keys.AuxInput3: 6,
        Keys.AuxInput4: 7,
        Keys.XNoise: 8,
        Keys.YNoise: 9,
        Keys.AuxOutput1: 10,
        Keys.AuxOutput2: 11,
        Keys.Phase: 12,
        Keys.SineOutputAmplitude: 13,
        Keys.DCLevel: 14,
        Keys.InternalFrequency: 15,
        Keys.ExternalFrequency: 16
    }

    blank = BoolCommand('DBLK')
    layout = DictCommand('DLAY', LayoutDict)
    config = DictIndexCommand('CDSP', ParameterDict, 3, 0, ChannelDict)
    graph_enable = BoolIndexCommand('CGRF', 3, 0, ChannelDict)

    def get_screen(self):
        raise NotImplemented('get_screen() is not implemented')


class Chart(Component):
    ChannelDict = Display.ChannelDict
    TimeDivDict = {
        0.5:       0,
        1.0:       1,
        2.0:       2,
        5.0:       3,
        10.0:      4,
        30.0:      5,
        60.0:      6,
        120.0:     7,
        300.0:     8,
        600.0:     9,
        1800.0:   10,
        3600.0:   11,
        7200.0:   12,
        21600.0:  13,
        43200.0:  14,
        86400.0:  15,
        172800.0: 16
    }
    CursorDisplayModeDict = {
        Keys.DateTime: 0,
        Keys.IntervalTime:  1
    }
    CursorReadoutModeDict = {
        Keys.Mean: 0,
        Keys.Maximum:  1,
        Keys.Minimum:  2
    }
    CursorWidthDict={
        Keys.Line:   0,
        Keys.Narrow: 1,
        Keys.Wide:   2
    }
    CursorValueDict = {
        Keys.Data1:   0,
        Keys.Data2:   1,
        Keys.Data3:   2,
        Keys.Data4:   3,
        Keys.Status:   4
    }

    time_division = DictCommand('GSPD', TimeDivDict)
    vertical_division = FloatIndexCommand('GSCL', 3, 0, ChannelDict)
    vertical_offset = FloatIndexCommand('GOFF', 3, 0, ChannelDict)
    enable = BoolIndexCommand('CGRF', 3, 0, ChannelDict)
    live = BoolCommand('GLIV')
    cursor_position = IntCommand('PCUR')
    cursor_relative = BoolCommand('CURREL')
    cursor_display_mode = DictCommand('CURDISP', CursorDisplayModeDict)
    cursor_readout_mode = DictCommand('CURBUG', CursorReadoutModeDict)
    cursor_width = DictCommand('FCRW', CursorWidthDict)
    cursor_value = IntIndexGetCommand('SCRY', 4, 0, CursorValueDict)
    cursor_date_time = GetCommand('CURDATTIM')
    cursor_interval_time = GetCommand('CURINTERVAL')

    def auto_scal(self, channel):
        self.comm.send(f'GACT {channel}')

    def auto_find(self, channel):
        self.comm.send(f'GAUF {channel}')


class FFT(Component):
    SourceDict = {
        Keys.ADC:    0,
        Keys.MIXER:  1,
        Keys.FILTER: 2
    }

    DBDivDict = {}
    index = -9
    for i in range(6):
        for j in [1.0, 2.0, 5.0]:
            DBDivDict[float(f'{(j * 10 ** (i - 3)):.1e}')] = index
            index += 1

    AverageDict = {
        1:   0,
        3:   1,
        10:  2,
        30:  3,
        100: 4
    }

    source = DictCommand('FFTR', SourceDict)
    vertical_scale = DictCommand('FFTS', DBDivDict)
    vertical_offset = FloatCommand('FFTO')
    max_span = FloatGetCommand('FFTMAXSPAN')
    span = FloatCommand('FFTSPAN')
    average = DictCommand('FFTA', AverageDict)
    live = BoolCommand('FFTL')
    cursor_width = DictCommand('FCRW', Chart.CursorWidthDict)
    cursor_frequency = FloatGetCommand('FCRX')
    cursor_value = FloatGetCommand('FCRY')

    def auto_scale(self):
        self.comm.send('FAUT')


class Scan(Component):
    ParameterDict = {
        Keys.InternalFrequency:  0,
        Keys.ReferenceAmplitude: 1,
        Keys.ReferenceOffset:    2,
        Keys.AuxOutput1:          3,
        Keys.AuxOutput2:          4
    }
    ScaleDict = {
        Keys.Linear: 0,
        Keys.Log:    1
    }
    EndDict = {
        Keys.Once:    0,
        Keys.Repeat:  1,
        Keys.UpDown: 2
    }
    AttenuationDict = {
        Keys.Auto:  0,
        Keys.Fixed: 1
    }
    IntervalDict = {
        0.008: 0,
        0.016: 1,
        0.031: 2,
        0.078: 3,
        0.155: 4,
        0.469: 5,
        0.938: 6,
        1.875: 7,
        4.688: 8,
        9.375: 9,
        28.12: 10,
        56.25: 11,
        112.5: 12,
        337.0: 13,
        675.0: 14,
        1350.0: 15,
        2700.0: 16
    }
    StateDict = {
        Keys.OFF:     0,
        Keys.RESET:   1,
        Keys.RUNNING: 2,
        Keys.PAUSED:  3,
        Keys.DONE:    4
    }
    RangeDict = {
        Keys.BEGIN: 0,
        Keys.END:   1
    }

    parameter = DictCommand('SCNPAR', ParameterDict)
    scale = DictCommand('SCNLOG', ScaleDict)
    end_mode = DictCommand('SCNEND', EndDict)
    period = FloatCommand('SCNSEC')
    amplitude_attenuation_mode = DictCommand('SCNAMPATTN', AttenuationDict)
    offset_attenuation_mode = DictCommand('SCNDCATTN', AttenuationDict)
    interval = DictCommand('SCNINRVL', IntervalDict)
    enable = BoolCommand('SCNENBL')
    state = DictGetCommand('SCNSTATE', StateDict)
    frequency_range = FloatIndexCommand('SCNFREQ', 1, 0, RangeDict)
    amplitude_range = FloatIndexCommand('SCNAMP', 1, 0, RangeDict)
    offset_range = FloatIndexCommand('SCNDC', 1, 0, RangeDict)
    aux_out1_range = FloatIndexCommand('SCNAUX1', 1, 0, RangeDict)
    aux_out2_range = FloatIndexCommand('SCNAUX2', 1, 0, RangeDict)

    def start(self):
        self.comm.send('SCNRUN')

    def pause(self):
        self.comm.send('SCNPAUSE')

    def reset(self):
        self.comm.send('SCNRST')


class DataTransfer(Component):
    ChannelDict = Display.ChannelDict
    ParameterDict = Display.ParameterDict
    channel_config = DictIndexCommand('CDSP', ParameterDict, 3, 0, ChannelDict)
    channel_value = FloatIndexGetCommand('OUTR', 3, 0, ChannelDict)
    value = FloatIndexGetCommand('OUTP', 16, 0, ParameterDict)

    def get_values(self, p1, p2, p3=None):
        if p3:
            reply = self.comm.query_text(f'SNAP? {p1}, {p2}, {p3}')
        else:
            reply = self.comm.query_text(f'SNAP? {p1}, {p2}')
            
        return list(map(float, reply.split(',')))

    def get_channel_values(self):
        reply = self.comm.query_text('SNAPD?')
        return list(map(float, reply.split(',')))


class DataCapture(Component):
    ChannelDict = {
        Keys.X:    0,
        Keys.XY:   1,
        Keys.RT:   2,
        Keys.XYRT: 3
    }
    RunModeDict={
        Keys.Once:       0,
        Keys.Continuous: 1
    }
    TriggerModeDict ={
        Keys.Immediate:        0,
        Keys.TriggerStart:     1,
        Keys.SamplePerTrigger: 2
    }
    CaptureStateBitDict = {
        Keys.InProgress: 0,
        Keys.Triggered:  1,
        Keys.Wrapped:    2
    }
    
    buffer_size_in_kilobytes = IntCommand('CAPTURELEN')
    config = DictCommand('CAPTURECFG', ChannelDict)
    max_rate = FloatGetCommand('CAPTURERATEMAX')
    rate_divisor_exponent = IntSetCommand('CAPTURERATE')
    rate = FloatGetCommand('CAPTURERATE')
    state = IntGetCommand('CAPTURESTAT')
    data_size_in_bytes = IntGetCommand('CAPTUREBYTES')
    data_size_in_kilobytes = IntGetCommand('CAPTUREPROG')

    def start(self, run_mode=0, trigger_mode=0):
        self.comm.send('CAPTURESTART {}, {}'.format(run_mode, trigger_mode))

    def stop(self):
        self.comm.send('CAPTURESTOP')

    def get_data(self, index):
        reply = self.comm.query_text(f'CAPTUREVAL? {index}')
        return list(map(float, reply.split(',')))
        
    def get_all_data(self):
        data_type = self.config
        final_index = self.data_size_in_kilobytes
        
        with self.comm.get_lock():
            self.comm._send(f'CAPTUREGET? 0, {final_index:d}')
            buffer = self.comm._read_binary(2)
            # buffer[0] should be 35            
            digits = buffer[1] - 48
            
            buffer += self.comm._read_binary(digits)
            offset = digits + 2
            buffer_size = int(buffer[2 : offset])
            buffer += self.comm._read_binary(buffer_size)
            
        data_size  = (len(buffer) - offset) // 4
        self.unpack_format = '>{}f'.format(data_size)
        vals = unpack_from(self.unpack_format, buffer, offset)
        if data_type == Keys.X:
            column = 1
        elif data_type == Keys.XY or data_type == 'RT':
            column = 2
        elif data_type == Keys.XYRT:
            column = 4
        else:
            ValueError('Invalid data type {} in get_all_data()'.format(data_type))
            
        row = len(vals) // column
        arr = np.transpose(np.reshape(vals, (row, column)))    
        return arr


class DataStreamBuffer:
    def __init__(self, size=10000000):
        self._data_buffer_size = size
        self._data_points = 0
        self.reset(size)

    def reset(self, size=10000000):
        self._data_buffer_size = size
        self._data_points = 0
        self.time = np.empty(self._data_buffer_size)
        self.x = np.empty(self._data_buffer_size)
        self.y = np.empty(self._data_buffer_size)
        self.r = np.empty(self._data_buffer_size)
        self.th = np.empty(self._data_buffer_size)

    def get_buffer_size(self):
        return self._data_buffer_size

    def get_data_size(self):
        return self._data_points

    def add_data_block(self, x, y, r, th):
        block_size = x.size
        init = self._data_points
        final = init + block_size

        final = init + block_size
        if final > self._data_buffer_size:
            raise IndexError('Data reached the data buffer size.')

        ti = np.arange(init, final)
        self.time[init: final] = ti
        self.x[init: final] = x
        self.y[init: final] = y
        self.r[init: final] = r
        self.th[init: final] = th
        self._data_points = final


class DataStream(Component):
    ChannelDict = DataCapture.ChannelDict
    FormatDict = {
        Keys.Float32: 0,
        Keys.Int16:   1
    }
    PacketSizeDict = {
        1024: 0,
        512:  1,
        256:  2,
        128:  3
    }
    OptionBitDict = {
        Keys.LittleEndian: 1,
        Keys.DataIntegrityChecking: 2
    }

    channel = DictCommand('STREAMCH', ChannelDict)
    max_rate = FloatGetCommand('STREAMRATEMAX')
    rate = IntCommand('STREAMRATE')
    format = DictCommand('STREAMFMT', FormatDict)
    packet_size = DictCommand('STREAMPCKT', PacketSizeDict)
    port = IntCommand('STREAMPORT')
    option = IntCommand('STREAMOPTION')
    enable = BoolCommand('STREAM')

    def __init__(self, parent, buffer_size=1000000):
        super().__init__(parent)
        self.data_buffer_size = buffer_size
        self.data = DataStreamBuffer(self.data_buffer_size)

    def _prepare(self):
        self.udp_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.udp_socket.bind(('', self.port))
        self.timeout = 10
        self.udp_socket.settimeout(self.timeout)
        self.prepared_channel = self.channel

        self.prepared_packet_size = self.packet_size
        self.unpack_format = '>{}h'.format(self.prepared_packet_size // 2) if self.format == 'int16' else\
                             '>{}f'.format(self.prepared_packet_size // 4)
        self.data.reset(self.data_buffer_size)

    def receive_packet(self):
        buffer, _ = self.udp_socket.recvfrom(self.prepared_packet_size + 4)
        header = unpack_from('>I', buffer)[0]
        packet_number = header & 0xff
        
        # from the manual page 172
        packet_content = (header >> 8) & 0x0f
        packet_size = (header >> 12) & 0x0f
        packet_rate = (header >> 16) & 0xff
        packet_status = (header >> 24) & 0xff

        vals = unpack_from(self.unpack_format, buffer, 4)

        arr = None
        if self.prepared_channel == Keys.X:
            arr = np.array(vals)

        elif self.prepared_channel == Keys.XY:
            rows = len(vals) // 2
            mat = np.transpose(np.reshape(vals, (rows, 2)))
            r = np.sqrt(np.square(mat[0]) + np.square(mat[1]))
            th = 180.0 / np.pi * np.arctan2(mat[1], mat[0])
            arr = np.append(np.append(mat, np.reshape(r, (1, rows)), axis=0),
                            np.reshape(th, (1,rows)), axis=0)

        elif self.prepared_channel == Keys.RT:
            rows = len(vals) // 2
            mat = np.transpose(np.reshape(vals, (rows, 2)))
            angle = np.pi / 180.0 * mat[1]
            x = mat[0] * np.cos(angle)
            y = mat[0] * np.sin(angle)
            arr = np.append(np.append(np.reshape(x, (1, rows)),
                                      np.reshape(y, (1, rows)), axis=0),
                            mat, axis=0)

        elif self.prepared_channel == Keys.XYRT:
            row = len(vals) // 4
            arr = np.transpose(np.reshape(vals, (row, 4)))
        else:
            raise ValueError(f'{self.prepared_channel} is not in ChannelDict')
        return arr, packet_number

    def start(self):
        self._prepare()
        self.enable = True

    def stop(self):
        self.enable = False
        self.udp_socket.close()


class Status(Component):
    SerialPollStatusBitDict = {
        Keys.ERR: 2,
        Keys.LIA: 3,
        Keys.MAV: 4,
        Keys.ESB: 5,
        Keys.SRQ: 6
    }
    EventStatusBitDict = {
        Keys.OPC: 0,
        Keys.INP: 1,
        Keys.QRY: 3,
        Keys.EXE: 4,
        Keys.CMD: 5,
        Keys.URQ: 6,
        Keys.PON: 7,
    }
    ErrorStatusBitDict = {
        Keys.CLK: 0,
        Keys.BACKUP: 1,
        Keys.VXI: 4,
        Keys.GPIB: 5,
        Keys.USBDEV: 6,
        Keys.USBHOST: 7
    }
    LiaStatusBitDict = {
        Keys.CH1OV:  0,
        Keys.CH2OV:  1,
        Keys.UNLK:   3,
        Keys.RANGE:  4,
        Keys.SYNCF:  5,
        Keys.SYNCOV: 6,
        Keys.TRIG:   7,
        Keys.DAT1OV: 8,
        Keys.DAT2OV: 9,
        Keys.DAT3OV: 10,
        Keys.DAT4OV: 11,
        Keys.DCAPFIN:12,
        Keys.SCNST:  13,
        Keys.SCNFIN: 14,
    }
    OverloadStatusBitDict = {
        Keys.Ch1OutputScale: 0,
        Keys.Ch2OutputScale: 1,
        Keys.ExternalReferenceUnlocked: 3,
        Keys.InputRange: 4,
        Keys.DataCh1Scale: 8,
        Keys.DataCh2Scale: 9,
        Keys.DataCh3Scale: 10,
        Keys.DataCh4Scale: 11,
    }

    event_enable = IntCommand('*ESE')
    event = IntGetCommand('*ESR')
    event_enable_bit = BoolIndexCommand('*ESE', 7, 0, EventStatusBitDict)
    event_bit = BoolIndexGetCommand('*ESR', 7, 0, EventStatusBitDict)

    serial_poll_enable = IntCommand('*SRE')
    serial_poll = IntGetCommand('*STB')
    serial_poll_enable_bit = BoolIndexCommand('*SRE', 7, 0, SerialPollStatusBitDict)
    serial_poll_bit = BoolIndexGetCommand('*STB', 7, 0, SerialPollStatusBitDict)

    power_on_status_clear_bit = BoolCommand('*PSC')

    error_enable = IntCommand('ERRE')
    error = IntGetCommand('ERRS')
    error_enable_bit = BoolIndexCommand('ERRE', 7, 0, ErrorStatusBitDict)
    error_bit = BoolIndexGetCommand('ERRS', 7, 0, ErrorStatusBitDict)

    lock_in_enable = IntCommand('LIAE')
    lock_in = IntGetCommand('LIAS')
    lock_in_enable_bit = BoolIndexCommand('LIAE', 14, 0, LiaStatusBitDict)
    lock_in_bit = BoolIndexGetCommand('LIAS', 14, 0, LiaStatusBitDict)

    overload = IntGetCommand('CUROVLDSTAT')

    exclude_capture = [event_enable_bit, serial_poll_enable_bit,
                       error_enable_bit, lock_in_enable_bit]

    def clear(self):
        self.comm.send('*CLS')

    def get_status_text(self):
        msg = ''
        status_byte = self.serial_poll
        if self.SerialPollStatusBitDict[Keys.ERR]:
            err = self.error
            for key, val in self.ErrorStatusBitDict.items():
                if 2 ** val & err:
                    msg += 'Error bit {}, {} is set, '.format(val, key)

        if self.SerialPollStatusBitDict[Keys.LIA]:
            lia = self.lock_in
            for key, val in self.LiaStatusBitDict.items():
                if 2 ** val & lia:
                    msg += 'LIA status bit {}, {} is set, '.format(val, key)

        if self.SerialPollStatusBitDict[Keys.ESB]:
            event = self.event
            for key, val in self.EventStatusBitDict.items():
                if 2 ** val & event:
                    msg += 'Event status bit {}, {} is set, '.format(val, key)

        ovld = self.overload
        if ovld:
            for key, val in self.OverloadStatusBitDict.items():
                if 2 ** val & ovld:
                    msg += 'Overload bit {}, {} is set, '.format(val, key)

        if msg == '':
            msg = 'OK'
        return msg



from srsgui.inst.exceptions import InstCommunicationError
from srsgui.inst.communications.interface import Interface

try:
    import pyvisa
    VISA_AVAILABLE = True
except (ImportError, ModuleNotFoundError):
    VISA_AVAILABLE = False


class VisaInterface(Interface):
    """Interface to use VISA"""

    NAME = 'visa'

    if VISA_AVAILABLE:
        rm = pyvisa.ResourceManager()
    else:
        rm = None

    def __init__(self):
        super(VisaInterface, self).__init__()
        self.type = VisaInterface.NAME
        self._visa = None
        self._resource_name = None
        self._is_connected = False

        if not VISA_AVAILABLE:
            raise ImportError("PyVisa is not installed. Install PyVisa to use VISA interface")

    def connect(self, resource):
        try:
            self._visa = self.rm.open_resource(resource)
            self._resource_name = resource
        except Exception as e:
            raise InstCommunicationError('Failed to connect {}'.format(resource))
        else:
            self._is_connected = True

            if self._connect_callback:
                self._connect_callback('Connected VISA: {}'
                                       .format(self._resource_name))

    def disconnect(self):
        self._visa.close()
        self._is_connected = False
        self._resource_name = None
        if self._disconnect_callback:
            self._disconnect_callback('Disconnected VISA: {}'.format(self._resource_name))

    @staticmethod
    def parse_parameter_string(param_string):
        connect_parameters = []
        params = param_string.split(':', 1)
        interface_type = params[0].strip().lower()
        if interface_type != VisaInterface.NAME:
            return None
        if len(params) <= 1:
            raise ValueError('Not enough parameters in "{}"'.format(param_string))
        else:
            connect_parameters.append(interface_type)  # 'visa'
            connect_parameters.append(params[1])  # resource string
        return connect_parameters

    def _send(self, cmd):
        self._visa.write(cmd)

    def _recv(self):
        reply = self._visa.read()
        return reply

    def _read_binary(self, length=-1):
        reply = self._visa.read_bytes(length)
        return reply

    def query_text(self, cmd):
        with self.get_lock():
            reply = self._visa.query(cmd)
        if self._query_callback:
            self._query_callback('Queried Cmd: {} Reply: {}'.format(cmd, reply))

        return reply

    def clear_buffer(self):
        self._visa.clear()

    def get_visa_instrument(self):
        return self._visa

    def get_info(self):
        return {'type': self.type,
                'resource_name': self._resource_name,
                }

    def set_timeout(self, timeout):
        self._timeout = timeout
        if self._visa:
            self._visa.timeout = self._timeout * 1000

    def get_timeout(self):
        return self._timeout

    @classmethod
    def find(cls):
        if not VISA_AVAILABLE:
            return ['No PyVisa installed']
        if not cls.rm:
            return []

        raw_items = cls.rm.list_resources()
        items = []
        for item in raw_items:
            if not item.startswith('ASRL'):
                items.append(item)
        return items






class Keys:
    # Reference component
    Auto = 'auto'
    Internal = 'internal'
    External = 'external'
    Common = 'common'
    Difference = 'difference'
    Dual = 'dual'
    Chop = 'chop'
    Sine = 'sine'
    PisitiveTTL = 'positive TTL'
    NegativeTTL = 'negative TTL'
    R50Ohms = '50 Ohms'
    R1Meg = '1 Meg'

    # Signal component
    Voltage = 'voltage'
    Current = 'current'
    AOnly = 'A'
    AMinusB = 'A-B'
    AC = 'AC'
    DC = 'DC'
    Float = 'float'
    Ground = 'ground'

    # Output component
    XY = 'XY'
    RT = 'RT'

    X = 'X'
    Y = 'Y'
    R = 'R'
    Theta = 'Theta'

    OutputCh1 = 'OCH1'
    OutputCh2 = 'OCH2'

    # Aux component
    Channel1 = 'CH1'
    Channel2 = 'CH2'
    Channel3 = 'CH3'
    Channel4 = 'CH4'

    # Display component
    Trend = 'trend'
    History = 'history'
    BarHistory = 'bar History'
    FFT = 'FFT'
    BarFFT = 'bar FFT'
    Bar8 = 'bar 8'

    Data1 = 'Data 1'
    Data2 = 'Data 2'
    Data3 = 'Data 3'
    Data4 = 'Data 4'

    AuxInput1 = 'aux in 1'
    AuxInput2 = 'aux in 2'
    AuxInput3 = 'aux in 3'
    AuxInput4 = 'aux in 4'

    XNoise = 'X noise'
    YNoise = 'Y noise'

    AuxOutput1 = 'aux out 1'
    AuxOutput2 = 'aux out 2'

    Phase = 'Phase'
    SineOutputAmplitude = 'amplitude'
    DCLevel = 'DC level'
    InternalFrequency = 'int. freq.'
    ExternalFrequency = 'ext. freq.'

    # Chart component
    DateTime = 'date time'
    IntervalTime = 'interval time'

    Mean = 'mean'
    Maximum = 'max'
    Minimum = 'min'

    Line = 'line'
    Narrow = 'narrow'
    Wide = 'wide'

    Status = 'status'

    # FFT component
    ADC = 'ADC'
    MIXER = 'mixer'
    FILTER = 'filter'

    # Scan component
    ReferenceAmplitude = 'reference amplitude'
    ReferenceOffset = 'reference offset'

    Linear = 'linear'
    Log = 'log'

    Once = 'once'
    Repeat = 'repeat'
    UpDown = 'up down'

    Fixed = 'fixed'

    OFF = 'off'
    RESET = 'reset'
    RUNNING = 'running'
    PAUSED = 'paused'
    DONE = 'done'

    BEGIN = 'begin'
    END = 'end'

    # DataCapture component
    XYRT = 'XYRT'

    Continuous = 'continuous'
    Immediate = 'immediate'
    TriggerStart = 'trigger start'
    SamplePerTrigger = 'sample per trigger'

    InProgress = 'in progress'
    Triggered = 'triggered'
    Wrapped = 'wrapped'

    # DataStream component
    Float32 = 'float32'
    Int16 = 'int16'
    LittleEndian = 'little endian'
    DataIntegrityChecking = 'data checking'

    # Status component

    ERR = 'ERR'
    LIA = 'LIA'
    MAV = 'MAV'
    ESB = 'ESB'
    SRQ = 'SRQ'

    OPC = 'OPC'
    INP = 'INP'
    QRY = 'QRY'
    EXE=  'EXE'
    CMD = 'CMD'
    URQ = 'URQ'
    PON = 'PON'

    CLK =     'CLK'
    BACKUP =  'BACKUP'
    VXI =     'VXI'
    GPIB =    'GPIB'
    USBDEV =  'USBDEV'
    USBHOST = 'USBHOST'

    CH1OV = 'CH1OV'
    CH2OV = 'CH2OV'
    UNLK = 'UNLK'
    RANGE = 'RANGE'
    SYNCF = 'SYNCF'
    SYNCOV = 'SYNCOV'
    TRIG = 'TRIG'
    DAT1OV = 'DAT1OV'
    DAT2OV = 'DAT2OV'
    DAT3OV = 'DAT3OV'
    DAT4OV = 'DAT4OV'
    DCAPFIN = 'DCAPFIN'
    SCNST = 'SCNST'
    SCNFIN = 'SCNFIN'

    Ch1OutputScale = 'ch1 output scale'
    Ch2OutputScale = 'ch2 output scale'
    ExternalReferenceUnlocked = 'external reference unlocked'
    InputRange = 'input range'
    DataCh1Scale = 'data CH1 scale'
    DataCh2Scale = 'data cH2 scale'
    DataCh3Scale = 'data cH3 scale'
    DataCh4Scale = 'data cH4 scale'




from .vxi11interface import Vxi11Interface
from .visainterface import VisaInterface

from srsgui import Instrument
from srsgui.inst import TcpipInterface, SerialInterface
from srsgui.task.inputs import IntegerListInput, BoolInput, \
                               Ip4Input, IntegerInput, FindListInput

from .components import Reference, Signal, Output, Aux, Auto, \
                        Display, Chart, FFT, Scan,\
                        DataTransfer, DataCapture, DataStream,\
                        Status


class SR860(Instrument):
    _IdString = 'SR86[05][A]?,'  # Regular expression including SR860, SR865 and SR865A
    MaxFrequency = 5e5
    
    available_interfaces = [
        [
            Vxi11Interface,
            {
                'ip_address': Ip4Input('192.168.1.10'),
            }
        ],
        [
            VisaInterface,
            {
                'resource': FindListInput(),
            }
        ],
        [
            TcpipInterface,
            {
                'ip_address': Ip4Input('192.168.1.10'),
                'port': 23
            }
        ],
        [
            SerialInterface,
            {
                'port': FindListInput(),
                'baud_rate': IntegerListInput([9600, 19200, 38400, 115200,
                                               230400, 460800], 3)
            }
        ],
    ]
    
    def __init__(self, interface_type=None, *args):
        super().__init__(interface_type, *args)

        self.ref = Reference(self)
        self.signal = Signal(self)
        self.output = Output(self)
        self.aux = Aux(self)
        self.auto = Auto(self)
        self.display = Display(self)
        self.chart = Chart(self)
        self.fft = FFT(self)
        self.scan = Scan(self)
        self.data = DataTransfer(self)
        self.capture = DataCapture(self)
        self.stream = DataStream(self)
        self.status = Status(self)
        
    def connect(self, interface_type, *args):
        super().connect(interface_type, *args)
        if isinstance(self.comm, TcpipInterface):
            print(self.query_text(''))  # Read out the initial string

    def reset(self):
        self.send('*RST')

    def get_status(self):
        return self.status.get_status_text()


#!/usr/bin/env python3

import sys
from pathlib import Path
from srsgui.ui.qt.QtWidgets import QApplication
from srsgui.ui.taskmain import TaskMain


def main():
    TaskMain.ApplicationName = 'sr860'
    TaskMain.DefaultConfigFile = str(Path(__file__).parent /
        "{}.taskconfig".format(TaskMain.ApplicationName))
    app = QApplication(sys.argv)
    main_window = TaskMain()
    main_window.show()
    app.exec_()


if __name__ == '__main__':
    main()





import time
import logging

from srsgui import Task
from srsgui import BoolInput, IntegerListInput, FloatListInput, InstrumentInput, CommandInput
from srsinst.sr860 import SR860, get_sr860

from srsinst.sr860.instruments.components import Signal
from srsinst.sr860.instruments.keys import Keys


class SignalToCurrentModeTask(Task):
    """
When the task is selected, the relevant parameters are read from the unit, \
and updates the input panel display. The values in the unit will change, when the \
Apply button is pressed.

When this task runs, it sets the input mode to the current inout mode.
    """
    InstName = 'inst to change'
    InputGain = 'input gain (Ohm)'
    Sensitivity = 'sensitivity (A)'
    TimeConstant = 'time constant (s)'
    FilterSlope = 'filter slope (dB/oct)'
    SyncFilter = 'synchronous filter'
    AdvancedFilter = 'advanced filter'

    input_parameters = {
        InstName:       InstrumentInput(),
        InputGain:      CommandInput('signal.current_input_gain', Signal.current_input_gain),
        Sensitivity:    CommandInput('signal.current_sensitivity', Signal.current_sensitivity),
        TimeConstant:   CommandInput('signal.time_constant', Signal.time_constant),
        FilterSlope:    CommandInput('signal.filter_slope', Signal.filter_slope),
        SyncFilter:     CommandInput('signal.sync_filter', Signal.sync_filter),
        AdvancedFilter: CommandInput('signal.advanced_filter', Signal.advanced_filter)
    }

    def setup(self):
        self.logger = logging.getLogger(__file__)
        self.lockin = get_sr860(self, self.get_input_parameter(self.InstName))

    def test(self):
        self.lockin.signal.input_mode = Keys.Current
        self.params = self.get_all_input_parameters()
        self.logger.info(self.params)
        self.logger.info('Input mode change to the current mode')

    def cleanup(self):
        pass



import time
import numpy as np


class TwoByTwoShareXPlot:
    """Plot for SR865A data streaming
    """
    def __init__(self, fig, data_buffer):
        self.figure = fig
        self.data = data_buffer
        self.update_rate = 10  # Hz
        if self.update_rate > 25:
            self.update_rate = 25

        self.update_period = 1.0 / self.update_rate
        self.max_points_in_plot = 10000

        self.ax = self.figure.subplots(nrows=2, ncols=2, sharex=True)
        li00, = self.ax[0][0].plot(self.data.time[:2], self.data.x[:2], color='#00d000')
        li01, = self.ax[0][1].plot(self.data.time[:2], self.data.y[:2], color='#00e0e0')
        li10, = self.ax[1][0].plot(self.data.time[:2], self.data.r[:2], color='#e0e000')
        li11, = self.ax[1][1].plot(self.data.time[:2], self.data.th[:2], color='orange')
        self.line = [li00, li01, li10, li11]

        self.xlim_min = 0.0
        self.xlim_max = 10000
        self.ax[0][0].set_xlim(self.xlim_min, self.xlim_max)
        self.ax[0][0].set_ylim(-5, 5)
        self.ax[0][1].set_ylim(-5, 5)
        self.ax[1][0].set_ylim(-5, 5)
        self.ax[1][1].set_ylim(-200, 200)

        self.ax[0][0].set_title('X')
        self.ax[0][1].set_title('Y')
        self.ax[1][0].set_title('R')
        self.ax[1][1].set_title('Theta')

        self.index_min = 0
        self.index_max = 1
        self.index_step = 1

        self.ax[0][0].callbacks.connect('xlim_changed', self.on_xlim_changed)
        self.ax[0][1].callbacks.connect('xlim_changed', self.on_xlim_changed)
        self.ax[1][0].callbacks.connect('xlim_changed', self.on_xlim_changed)
        self.ax[1][1].callbacks.connect('xlim_changed', self.on_xlim_changed)

        self.init_time = time.time()
        self.last_time = self.init_time

    def on_xlim_changed(self, event_ax):
        x_min, x_max = event_ax.get_xlim()
        self.xlim_min = int(x_min)
        self.xlim_max = int(x_max) + 1

        self.index_step = (self.xlim_max - self.xlim_min) // self.max_points_in_plot
        if self.index_step < 1:
            self.index_step = 1
        data_points = self.data.get_data_size()
        self.index_min = self.xlim_min if self.xlim_min > 0 else 0
        self.index_max = self.xlim_max if self.xlim_max < data_points else data_points

    def request_plot_update(self):
        current_time = time.time()
        if current_time - self.last_time < self.update_period:
            return False

        data_size = self.data.get_data_size()
        if data_size < self.xlim_max:
            self.index_max = data_size

        s = slice(self.index_min, self.index_max, self.index_step)
        ti = self.data.time[s]
        self.line[0].set_data(ti, self.data.x[s])
        self.line[1].set_data(ti, self.data.y[s])
        self.line[2].set_data(ti, self.data.r[s])
        self.line[3].set_data(ti, self.data.th[s])

        self.last_time = current_time
        return True

    def init_data_save(self, task):
        """
        Prepare to save streaming data to the file from the task
        """
        if not (hasattr(task, 'session_handler') and task.session_handler):
            raise ValueError('task has no session_handler')

        self.last_data_point = 0
        self.data_table_name = 'Stream Data'
        # Create a table to dave data
        task.create_table_in_file(self.data_table_name,
                                  'Time', 'X', 'Y')

    def save_data(self, task):
        """
        Save streaming data to file
        """
        data_index = self.last_data_point
        self.data_points = self.data.get_data_size()

        while data_index < self.data_points:
            task.add_to_table_in_file('Stream Data',
                self.data.time[data_index],
                self.data.x[data_index],
                self.data.y[data_index])
            data_index += 1
        self.last_data_point = self.data_points



import time
import logging

from srsgui import Task
from srsgui import BoolInput, IntegerListInput, FloatListInput, InstrumentInput, CommandInput
from srsinst.sr860 import SR860, get_sr860

from srsinst.sr860.instruments.components import Scan
from srsinst.sr860.instruments.keys import Keys

from srsinst.sr860.tasks.scanfrequencytask import ScanFrequencyTask


class ScanRefOffsetTask(ScanFrequencyTask):
    """

    """
    ScanName = 'Reference Offset'
    ScanParameter = Keys.ReferenceAmplitude
    ScanAttenuationCommand = 'scan.offset_attenuation_mode'
    ScanAttenuationInstance = Scan.offset_attenuation_mode
    ScanBeginCommand = 'scan.offset_range["begin"]'
    ScanEndCommand = 'scan.offset_range["end"]'
    ScanCommandInstance = Scan.offset_range

    InstName = 'inst to change'
    ScanScale = 'Scan Scale'
    ScanEndMode = 'End Mode'
    ScanAttenuation = 'Attenuation Mode'
    ScanPeriod = 'Scan period (s)'
    ScanInterval = 'Scan Interval (s)'
    ScanBegin = 'Scan Begin Value (V)'
    ScanEnd = 'Scan End Value (V)'

    input_parameters = {
        InstName:     InstrumentInput(),
        ScanScale:    CommandInput('scan.scale', Scan.scale),
        ScanEndMode:  CommandInput('scan.end_mode', Scan.end_mode),
        ScanPeriod:   CommandInput('scan.period', Scan.period),
        ScanAttenuation: CommandInput(ScanAttenuationCommand, ScanAttenuationInstance),
        ScanInterval: CommandInput('scan.interval', Scan.interval),
        ScanBegin:    CommandInput(ScanBeginCommand, ScanCommandInstance),
        ScanEnd:      CommandInput(ScanEndCommand, ScanCommandInstance),
    }




import time
import logging
import socket
import struct
import threading

from srsgui import Task
from srsgui import IntegerInput, ListInput
from srsinst.sr860 import SR860, get_sr860


class SidnTask(Task):
    """
    Check the time taken for 100 *IDN queries
    to check communication speed
    """

    input_parameters = {}
    Repeat = 100

    def setup(self):
        self.logger = logging.getLogger(__file__)
        self.lockin = get_sr860(self)
        
        print(self.lockin.query_text('*idn?'))

        # Mark the time 0
        self.init_time = time.time()

    def test(self):
        start_time = time.time()
        for i in range(self.Repeat):
            print(self.lockin.query_text('*idn?'))
        finish_time = time.time()
        print('Time for {} *idn: {}'.format(self.Repeat, finish_time - start_time))

    def cleanup(self):
        pass




import time
import logging

from srsgui import Task
from srsgui import InstrumentInput, ListInput
from srsinst.sr860 import SR860, get_sr860

from srsinst.sr860.instruments.components import Aux
from srsinst.sr860.instruments.keys import Keys


class AutoFunctionTask(Task):
    """
    Run one of the auto functions: auto phase, auto range, or auto scale.
    """

    InstName = 'inst to change'
    FunctionSelection = 'Select Auto function to run'
    Functions = ['Phase', 'Range', 'Scale']

    input_parameters = {
        InstName:       InstrumentInput(),
        FunctionSelection: ListInput(Functions)
    }

    def setup(self):
        self.logger = logging.getLogger(__file__)
        self.lockin = get_sr860(self, self.get_input_parameter(self.InstName))
        self.params = self.get_all_input_parameters()

    def test(self):
        chosen = self.params[self.FunctionSelection]
        if chosen == 'Phase':
            self.lockin.auto.set_phase()
        elif chosen == 'Range':
            self.lockin.auto.set_range()
        elif chosen == 'Scale':
            self.lockin.auto.set_scale()
        else:
            self.logger.error('unknown option "{}"'.format(chosen))

        self.logger.info(self.params)
        self.logger.info('Auto {} has run.'.format(chosen))

    def cleanup(self):
        pass



import time
import logging

from srsgui import Task
from srsgui import BoolInput, IntegerListInput, FloatListInput, InstrumentInput, CommandInput
from srsinst.sr860 import SR860, get_sr860

from srsinst.sr860.instruments.components import Scan
from srsinst.sr860.instruments.keys import Keys


class ScanFrequencyTask(Task):
    """

    """
    ScanName = 'Internal frequency'
    ScanParameter = Keys.InternalFrequency
    ScanBeginCommand = 'scan.frequency_range["begin"]'
    ScanEndCommand = 'scan.frequency_range["end"]'
    ScanCommandInstance = Scan.frequency_range

    InstName = 'inst to change'
    ScanScale = 'Scan Scale'
    ScanEndMode = 'End Mode'
    ScanPeriod = 'Scan period (s)'
    ScanInterval = 'Scan Interval (s)'
    ScanBegin = 'Scan Begin Value (Hz)'
    ScanEnd = 'Scan End Value (Hz)'

    input_parameters = {
        InstName:     InstrumentInput(),
        ScanScale:    CommandInput('scan.scale', Scan.scale),
        ScanEndMode:  CommandInput('scan.end_mode', Scan.end_mode),
        ScanPeriod:   CommandInput('scan.period', Scan.period),
        ScanInterval: CommandInput('scan.interval', Scan.interval),
        ScanBegin:    CommandInput(ScanBeginCommand, ScanCommandInstance),
        ScanEnd:      CommandInput(ScanEndCommand, ScanCommandInstance),
    }

    def setup(self):
        self.logger = logging.getLogger(__file__)
        self.lockin = get_sr860(self, self.get_input_parameter(self.InstName))

    def test(self):
        self.params = self.get_all_input_parameters()
        self.logger.info(self.params)
        self.lockin.scan.parameter = self.ScanParameter
        self.lockin.scan.enable = True
        self.logger.info('Scan mode change to {}'.format(self.ScanName))

    def cleanup(self):
        pass




from .instruments.vxi11interface import Vxi11Interface
from .instruments.visainterface import VisaInterface

from .instruments.sr860 import SR860
from .instruments.sr865 import SR865
from .instruments.sr865a import SR865A

from .instruments.get_instruments import get_sr860, get_sr865, get_sr865a

__version__ = "0.0.10"


import time
import math
import numpy as np
from srsgui import Task
from srsinst.sr860.plots.twobytwosharexplot import TwoByTwoShareXPlot
from srsinst.sr860.instruments.components import DataStreamBuffer


class SimulatedPlotTask(Task):
    input_parameters={}

    def setup(self):
        # storage for data generation
        self.dr = 1e-3
        self.dt = 1e-3

        self.figure = self.get_figure()
        self.data = DataStreamBuffer(10000000)
        self.plot = TwoByTwoShareXPlot(self.figure, self.data)

    def test(self):
        while True:
            if not self.is_running():
                break

            r, th = self.calc_chunk()
            time.sleep(0.001)

            x = r * np.sin(th)
            y = r * np.cos(th)

            self.data.add_data_block(x, y, r, th)
            self.notify_data_available()  # parent calls update() method

    def update(self, data):
        if self.plot.request_plot_update():
            self.request_figure_update()

    def cleanup(self):
        pass

    def calc_chunk(self):
        block_size = 256
        r = np.empty(block_size)
        t = np.empty(block_size)
        for i in range(block_size):
            self.dr += 1e-4
            self.dt += 1e-3
            r[i] = 2 + math.sin(self.dr)
            t[i] = math.pi / 2.0 + 1.0 * math.sin(self.dt)
        return r, t



import logging
from srsgui.task import Task
from .sr860 import SR860
from .sr865 import SR865
from .sr865a import SR865A

logger = logging.getLogger(__name__)


def get_sr860(task: Task, name=None) -> SR860:
    """
    Instead of using task.get_instrument() directly in a Task subclass,
    Defining a wrapper function with a instrument return type will help
    a context-sensitive editors display  attributes available
    for the instrument class.
    """
    if name is None:
        inst = list(task.inst_dict.values())[0]
    else:
        inst = task.get_instrument(name)

    if issubclass(type(inst), SR860):
        return inst
    else:
        logger.error('{} is not {}'.format(type(inst), SR860))
        return None


def get_sr865(task: Task, name=None) -> SR865:
    if name is None:
        inst = list(task.inst_dict.values())[0]
    else:
        inst = task.get_instrument(name)

    if issubclass(type(inst), SR865):
        return inst
    else:
        logger.error('{} is not {}'.format(type(inst), SR865))
        return None


def get_sr865a(task: Task, name=None) -> SR865A:
    if name is None:
        inst = list(task.inst_dict.values())[0]
    else:
        inst = task.get_instrument(name)

    if issubclass(type(inst), SR865A):
        return inst
    else:
        logger.error('{} is not {}'.format(type(inst), SR865A))
        return None




import logging
import vxi11

from srsgui.inst import Interface
from srsgui.inst import InstCommunicationError

logger = logging.getLogger(__file__)


class Vxi11Interface(Interface):
    
    NAME = 'vxi11'

    def __init__(self):
        super().__init__()
        self.type = Vxi11Interface.NAME
        self._vxi = None
        self._ip_address = ''
        self._timeout = 20
        
    def connect(self, ip_address):
        try:
            self._vxi = vxi11.Instrument(ip_address)
            print(self._vxi.ask('*IDN?'))
            self._ip_address = ip_address
            self._is_connected = True
            if self._connect_callback:
                self._connect_callback('Connected VXI11 to {}'
                                       .format(self._ip_address))

        except Exception as e:
            logger.error(e)

    def disconnect(self):
        self._vxi.close()
        self._is_connected = False
        if self._disconnect_callback:
            self._disconnect_callback('Disconnected VXI11 from : {}'.format(self._ip_address))

    @staticmethod
    def parse_parameter_string(param_string):
        connect_parameters = []
        params = param_string.split(':')
        num = len(params)
        interface_type = params[0].strip().lower()
        if interface_type != Vxi11Interface.NAME:
            return None
        if num > 2:
            raise ValueError('Too many parameters in "{}"'.format(param_string))
        if num > 1:
            connect_parameters.append(interface_type)  # 'serial'
            connect_parameters.append(params[1])  # IP address
        return connect_parameters

    def _send(self, cmd):
        self._vxi.write(cmd)

    def _recv(self):
        reply = self._vxi.read()
        return reply

    def _read_binary(self, length=-1):
        """
        Read a fixed number of bytes. VXI11 read_raw returns all the data contained
        in the last packet that covers the length of data.
        It could return larger than the specified size of data.
        """
        reply = self._vxi.read_raw(length)
        return reply

    def query_text(self, cmd):
        with self.get_lock():
            reply = self._vxi.ask(cmd)
        if self._query_callback:
            self._query_callback('Queried Cmd: {} Reply: {}'.format(cmd, reply))

        return reply

    def set_timeout(self, timeout):
        self._timeout = timeout
        if self._vxi:
            self._vxi.timeout = timeout * 1000

    def get_timeout(self):
        return self._timeout

    def clear_buffer(self):
        self._vxi.clear()

    def get_info(self):
        return {'type': self.type,
                'ip_address': self._ip_address,
                }




import time
import logging

from srsgui import Task
from srsgui import IntegerInput, FloatInput, ListInput, IntegerListInput

from srsinst.sr860 import SR860, get_sr860
from srsinst.sr860.instruments.components import DataStream

from srsinst.sr860.plots.twobytwosharexplot import TwoByTwoShareXPlot


class StreamingTask(Task):
    Duration = 'duration'
    Channels = 'Channels'
    DataFormat = 'data format'
    PacketSize = 'packet size'
    Rate = 'rate divider'
    Port = 'udp port'

    input_parameters = {
        Duration: IntegerInput(3600, ' s', 1, 360000, 1),
        Channels: ListInput(list(DataStream.ChannelDict.keys()), 1),
        DataFormat: ListInput(list(DataStream.FormatDict.keys())),
        PacketSize: IntegerListInput([1024, 512, 256, 128]),
        Rate: IntegerInput(4, '  (2^n) ', 0, 20, 1),
        Port: IntegerInput(1865, '', 1024, 65535, 1)
    }

    def setup(self):
        self.logger = logging.getLogger(__file__)
        self.lia = get_sr860(self)
        print(self.lia.query_text('*idn?'))

        self.lia.stream.enable = False
        self.lia.stream.option = 2
        self.lia.stream.channel = self.input_parameters[self.Channels].text
        self.lia.stream.format = self.input_parameters[self.DataFormat].text
        self.lia.stream.packet_size = self.input_parameters[self.PacketSize].get_value()
        self.lia.stream.rate = self.get_input_parameter(self.Rate)
        self.lia.stream.port = self.get_input_parameter(self.Port)

        self.duration_value = self.get_input_parameter(self.Duration)
        self.max_rate = self.lia.stream.max_rate
        self.sample_rate = self.max_rate / 2 ** self.lia.stream.rate
        self.packet_size = self.lia.stream.packet_size
        self.logger.info('Channels: {}, Data format: {}, Packet_size: {}, Rate: {:.3f} Hz, Port: {}'
                         .format(self.lia.stream.channel,
                                 self.lia.stream.format,
                                 self.packet_size,
                                 self.sample_rate,
                                 self.lia.stream.port,
                                 )
                         )

        self.plot = TwoByTwoShareXPlot(self.figure, self.lia.stream.data)

        # Mark the time 0
        self.init_time = time.time()

    def test(self):
        if self.get_input_parameter(self.Channels) == 0:
            raise ValueError('Channel X is not allowed,Choose other multiple channels')

        self.last_p_id = 0
        self.lia.stream.start()
        try:
            while time.time() - self.init_time < self.duration_value:
                if not self.is_running():
                    break

                block, p_id = self.lia.stream.receive_packet()
                self.lia.stream.data.add_data_block(*block)

                if self.last_p_id and p_id - self.last_p_id > 1:
                    self.logger.warning('{} missing packet(s) before ID:{}'
                                        .format(p_id - self.last_p_id - 1, p_id))
                self.last_p_id = p_id
                self.notify_data_available()
        except Exception as e:
            self.logger.error(e)

    def update(self, data):
        if self.plot.request_plot_update():
            self.request_figure_update()

    def cleanup(self):
        self.lia.stream.stop()
